<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://dialectpro.org/
 * @since      3.0.0
 *
 * @package    Dictionary_Pro
 * @subpackage Dictionary_Pro/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Dictionary_Pro
 * @subpackage Dictionary_Pro/admin
 * @author     Anne Dorko <hello@dialectpro.org>
 */
class Dictionary_Pro_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The version of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      array    $dictionaries    The dictionaries registered by this plugin.
	 */
	private $dictionaries;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    3.0.0
	 * @param      string $plugin_name       The name of this plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $dictionaries ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->dictionaries = $dictionaries;
		$this->slugs = array();
		if ( ! empty( $dictionaries ) ) {
			foreach ( $dictionaries as $dictionary ) {
				$this->slugs[] = $dictionary->slug();
			}
		}
	}

	public function register_license_page() {
		$dpro_license = get_option( 'dictionary_pro_license' );
		// Step 1: Redirect if license key is here
		if ( isset( $dpro_license['license_key'] ) && ! empty( $dpro_license['license_key'] ) ) {
			$this->handle_activation_redirect();
		} else {
			// Step 2: Check and update incoming data if no license key exists
			$error = '';
			if ( $_REQUEST && isset( $_REQUEST['license_key'] ) ) {
				$error = $this->activate_dictionary();
				if ( '' === $error ) {
					$this->handle_activation_redirect();
				}
			}
			// Step 3: If we're still reading code by now, load the activation page
			foreach ( $this->dictionaries as $dictionary ) {
				add_submenu_page(
					$dictionary->settings()->url(),
					__( 'Activation for Dictionary Pro', 'dictionary-pro' ),
					__( 'Enter License Key', 'dictionary-pro' ),
					'manage_options',
					'activation',
				array( $this, 'show_activation_page' ) );
			}
		}
	}

	function handle_activation_redirect() {
		global $pagenow, $post_type;
		foreach ( $this->dictionaries as $dictionary ) {
			// Goal: /wp-admin/edit.php?post_type=dictionary&page=activation
			if ( 'edit.php' == $pagenow &&
						null == $post_type && isset( $_GET['page'] ) &&
						'activation' == $_GET['page'] ) {
				wp_redirect( admin_url( '/edit.php?post_type=' . $dictionary->slug(), 'http' ), 301 );
				exit;
			}
		}
	}

	/**
	 * Handle pre-activation
	 */
	public function activate_dictionary() {
		global $version, $error;
		$version = $this->version;
		$error = '';
		// Check for form submission
		if ( $_REQUEST && isset( $_REQUEST['license_key'] ) ) {
			// Basic validation
			if ( empty( $_REQUEST['license_key'] ) ) {
				$error = __( 'Please enter a license key.', 'dictionary-pro' );
			}
			if ( empty( $_REQUEST['license_email'] ) ) {
				$error = __( 'Please enter the email registered with Dialect Pro.', 'dictionary-pro' );
			}
			if ( empty( $_REQUEST['license_password'] ) ) {
				$error = __( 'Please do not delete the hidden license password.', 'dictionary-pro' );
			}
			if ( empty( $error ) ) {
				$license_key = $_REQUEST['license_key'];
				$license_email = $_REQUEST['license_email'];
				$license_pw = $_REQUEST['license_password'];

				$api_params = array(
					'request' 	=> 'activation',
					'email'		=> $license_email,
					'licence_key' => $license_key,
					'product_id' => DICTIONARY_PRO_REFERENCE,
					'platform'	=> $this->remove_http( get_bloginfo( 'url' ) ),
					'instance'	=> $license_pw,
					'software_version'	=> $version,
				);

				$query = esc_url_raw( add_query_arg( $api_params, DICTIONARY_PRO_API_SERVER ) );
				$response = wp_remote_get( $query, array( 'sslverify' => true ) );

				if ( is_wp_error( $response ) ) {
		            $error = __( 'Unexpected error! The query returned with an error. Please email us at hello@dialectpro.org', 'dictionary-pro' );
		        } else {
					$license_data = json_decode( wp_remote_retrieve_body( $response ) );
					if ( null === $license_data ) {
						$error = 'There was an error connecting to the server, try again in a few minutes.';
					} elseif ( isset( $license_data->activated ) && 'inactive' === $license_data->activated ) {
						$error = '';
						if ( isset( $license_data->error ) ) {
							$error = $license_data->error . ' ';
						}
						if ( isset( $license_data->{'additional info'} ) ) {
							$error .= $license_data->{'additional info'};
						}
					} elseif ( isset( $license_data->activated ) && true == $license_data->activated ) {
						// Success!
						$dpro_license = get_option( 'dictionary_pro_license' );
						$dpro_license['license_key'] = $license_key;
						$dpro_license['license_email'] = $license_email;
						update_option( 'dictionary_pro_license', $dpro_license );
						// wp_redirect( admin_url( '/edit.php?post_type=dictionary' ) );
						// exit;
					} else {
						$error = '';
						if ( isset( $license_data->error ) ) {
							$error = $license_data->error . ' ';
						}
						if ( isset( $license_data->{'additional info'} ) ) {
							$error .= $license_data->{'additional info'};
						}
					}
				}
			}
		}
		return $error;
	}
	function show_activation_page() {
		// Show the page
		include WPD_DIRECTORY . '/admin/partials/activate.php';
	}

	public function remove_http( $url ) {
		$disallowed = array( 'http://', 'https://' );
		foreach ( $disallowed as $d ) {
			if ( strpos( $url, $d ) === 0 ) {
				return str_replace( $d, '', $url );
			}
		}
		return $url;
	}

	public function redirect_post_type_request() {
		// Avoid showing admin errors for a deactivated plugin, instead redirect users to the activation page
		if ( ! empty( $_GET ) && isset( $_GET['post_type'] ) && 'dictionary' === $_GET['post_type'] ) {
			if ( true === post_type_exists( 'dictionary' ) ) {
				return;
			} else {
				wp_redirect( admin_url( '/admin.php?page=activate-dictionary' ) );
				exit;
			}
		}
	}

	/**
	 * Add the Dictionary Pro menu Walker for dictionary menus
	 *
	 * @since 3.1.0
	 *
	 * @param  array $args argument array for a menu_item_parent
	 * @return array       args with Dictionary Pro walker for dictionary menus
	 */
	function dictionary_section_menu( $args ) {
		if ( is_object( $args['menu'] ) ) { return $args; }
		foreach ( $this->dictionaries as $dictionary ) {
			$settings = $dictionary->options();
			if ( isset( $settings['disable_empty_menus'] ) && true == $settings['disable_empty_menus'] ) {
				$options = strtolower( $args['theme_location'] . ' ' . $args['menu'] );
				if ( false !== strpos( $options, $dictionary->slug() ) || false !== strpos( $options, 'dictionaries' ) ) {
					$args['walker'] = new DictionaryPro_Walker_Nav_Menu();
				}
			}
		}
		return $args;
	}

	/**
	 * Welcome Screen functions
	 */
	public function redirect_to_welcome_page() {
		if ( isset( $_REQUEST['reset-dictionary-welcome'] ) && 'true' === $_REQUEST['reset-dictionary-welcome'] ) {
			delete_transient( '_has_viewed_dictionary_welcome_version_' . $this->version );
		}
		if ( ! empty( $_GET ) && isset( $_GET['post_type'] ) && 'dictionary' === $_GET['post_type'] ) {
			if ( true === post_type_exists( 'dictionary' ) ) {
				$check = get_transient( '_has_viewed_dictionary_welcome_version_' . $this->version );
				if ( false == $check ) {
					wp_redirect( admin_url( '/admin.php?page=welcome-dictionary' ) );
					exit;
				}
			}
		}
		if ( ! empty( $_GET ) && isset( $_GET['page'] ) && 'welcome-dictionary' === $_GET['page'] ) {
			$check = get_transient( '_has_viewed_dictionary_welcome_version_' . $this->version );
			if ( true == $check ) {
				wp_redirect( admin_url( '/edit.php?post_type=dictionary' ) );
				exit;
			}
		}
	}
	public function register_welcome_screen() {
		$check = get_transient( '_has_viewed_dictionary_welcome_version_' . $this->version );
		if ( ! $check ) {
			add_dashboard_page( 'Welcome to Dictionary Pro', 'Dictionary Welcome', 'manage_options', 'welcome-dictionary', array( $this, 'show_welcome_screen' ) );
		}
	}
	public function show_welcome_screen() {
		global $version;
		$version = $this->version;
		set_transient( '_has_viewed_dictionary_welcome_version_' . $this->version, true, 182 * WEEK_IN_SECONDS );
		// Show the page
		include WPD_DIRECTORY . '/admin/partials/welcome.php';
	}

	public function admin_footer_text( $text ) {
		$screen = get_current_screen();
		if ( 'dictionary' === $screen->post_type || strpos( $screen->id, 'dictionary' ) ) {
			$text = '<span id="footer-thankyou">Enjoy this plugin? Make money as an affiliate for <a href="https://dialectpro.org/affiliates/" target="_blank">Dictionary Pro</a>.</span>';
		}
		echo $text;
	}

	/**
	 * Check for and run updates
	 */
	public function check_updates() {
		$license = get_option( 'dictionary_pro_license' );
		/**
		 * Developers, please note that the license is checked before
		 * updates will be sent, even if you disable the return below.
		 *
		 * Please email hello@dialectpro.org for assistance
		 */

		// No license, don't bother checking for updates
		if ( empty( $license ) ) { return;
		}

		$upgrade_url = DICTIONARY_PRO_API_SERVER;
		$plugin_name = WPD_BASENAME;
		$product_id  = DICTIONARY_PRO_REFERENCE;
		$instance 	 = DICTIONARY_PRO_SECRET_KEY;
		$api_key	 = $license['license_key'];
		$api_email	 = $license['license_email'];
		$renew_license_url = 'https://dialectpro.org/account/';
		$domain 	 = get_bloginfo( 'url' );
		$plugin_or_theme = 'plugin';

			new Dictionary_Pro_Updates( $upgrade_url, $plugin_name, $product_id, $api_key, $api_email, $renew_license_url, $instance, $domain, $this->version, $plugin_or_theme, 'dictionary-pro' );
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    3.0.0
	 */
	public function enqueue_styles() {
		// General plugin admin styles
		wp_enqueue_style(
			$this->plugin_name,
			plugin_dir_url( __FILE__ ) . 'css/dictionary-pro-styles.css',
		array(), $this->version, 'all' );

		// Specific UI for registered post types
		$screen = get_current_screen();
		if ( in_array( $screen->id, $this->slugs )  ) {
			wp_register_style(
				'jquery-ui-styles',
			'http://ajax.googleapis.com/ajax/libs/jqueryui/1.8/themes/base/jquery-ui.css' );
			wp_enqueue_style( 'jquery-ui-styles' );
		}

		// Specific UI for regular posts and pages editing
		if ( in_array( $screen->id, array( 'post', 'page' ) ) ) {
			if ( ! current_user_can( 'edit_posts' ) && ! current_user_can( 'edit_pages' ) ) {
		        return;
			}
			if ( get_user_option( 'rich_editing' ) !== 'true' ) {
				return;
			}
			wp_enqueue_style( 'wp-jquery-ui-dialog' );
		}

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    3.0.0
	 */
	public function enqueue_scripts() {
		$screen = get_current_screen();

		// Scripts for dealing with custom post types
		if ( in_array( $screen->id, $this->slugs ) ) {
			wp_enqueue_script(
				$this->plugin_name,
				plugin_dir_url( __FILE__ ) . 'js/dictionary-pro-edit-entry.js',
				array( 'jquery', 'jquery-ui-autocomplete' ),
				$this->version,
			false );
		}

		if ( 'dictionary_page_settings' === $screen->id ) {
			wp_enqueue_script(
				$this->plugin_name,
				plugin_dir_url( __FILE__ ) . 'js/dictionary-pro-settings.js',
				array( 'jquery', 'jquery-ui-sortable' ),
				$this->version,
			false );
			add_thickbox();
		}

		// Scripts for dealing with normal WordPress things
		if ( in_array( $screen->id, array( 'post', 'page' ) ) ) {
			if ( ! current_user_can( 'edit_posts' ) && ! current_user_can( 'edit_pages' ) ) {
		        return;
			}
			if ( get_user_option( 'rich_editing' ) !== 'true' ) {
				return;
			}
			wp_enqueue_script(
				$this->plugin_name,
				plugin_dir_url( __FILE__ ) . 'js/dictionary-pro-edit-post.js',
				array( 'jquery', 'jquery-ui-core', 'jquery-ui-dialog' ),
				$this->version,
			false );
			add_thickbox();
		}
	}

	public function setup_tinymce_hook() {
		if ( ! current_user_can( 'edit_posts' ) && ! current_user_can( 'edit_pages' ) ) {
	        return;
		}
		if ( get_user_option( 'rich_editing' ) !== 'true' ) {
			return;
		}

		add_action( 'media_buttons', array( $this, 'add_entry_button' ), 15 );
		add_action( 'wp_ajax_dictionary-pro-inline-entry', array( $this, 'ajax_add_entry_inline' ) );
	}

	/**
	 * Plugin links
	 *
	 * @param  array  $links Default links provided by WordPress
	 * @param  string $file  File basename of plugin
	 * @return array        Returns modified links
	 */
	public function plugin_links( $links, $file ) {
		if ( WPD_BASENAME === $file ) {
			array_pop( $links );
			$links[] = '<a href="https://dialectpro.org/docs/">' . __( 'Documentation', 'dictionary-pro' ) . '</a>';
		    $links[] = '<a href="https://dialectpro.org/support/">' . __( 'Support', 'dictionary-pro' ) . '</a>';
		    $links[] = '<a href="https://dialectpro.org/semantics/">' . __( 'Learn More', 'dictionary-pro' ) . '</a>';
		    // $links[] = '<a href="http://twitter.com/annedorko">Follow @AnneDorko</a>';
		}
		return $links;
	}

	/**
	 * Register widgets
	 */

	public function widgets() {
		// TODO: Make this work for a variety of widgets
		register_widget( 'Dictionary_Pro_Widget' );
	}

	/**
	 * Functions to Handle AJAX Calls
	 */

	// Used for Autocomplete Search when linking Entries
	public function ajax_search_linked_posts() {
		$term = $_GET['term'];
		$post_type = $_GET['post_type'];
		$uid = $_GET['name'];
		$suggestions = array();

		$results = get_posts( array( 's' => $term, 'post_type' => $post_type ) );

		foreach ( $results as $result ) {
			setup_postdata( $result );
			$suggestion = array();
			$suggestion['label'] = get_the_title( $result->ID );
			$suggestion['id'] = $result->ID;
			$suggestion['name'] = $result->post_name;

			$suggestions[] = $suggestion;
		}

		wp_reset_query();

		$response = json_encode( $suggestions );
		echo $response;
		exit();
	}

	// Used for creating inline Dictionary entries
	function ajax_add_entry_inline() {
		global $_POST;
		$post_data = $_POST;
		if ( ! isset( $post_data['post_title'] ) ) { return; }
		if ( ! isset( $post_data['post_content'] ) ) { return; }
		// TODO: Expand abilities to create entries on the fly for multiple dictionaries
		$dictionary = array_shift( $this->dictionaries );
		$post_data['post_type'] = $dictionary->slug();
		$post_data['post_status'] = 'publish';
		$new_post_id = wp_insert_post( $post_data );
		$response = array(
			'status' => '200',
			'message' => 'OK',
			'new_post_ID' => $new_post_id,
		);
		header( 'Content-Type: application/json; charset=utf-8' );
		echo json_encode( $response );
		exit;
	}

	// Used to delete the current license key
	function ajax_delete_license_key() {
		$dpro_license = get_option( 'dictionary_pro_license' );
		if ( ! empty( $dpro_license ) ) {
			// Set a unique password based on the URL if none is set
			unset( $dpro_license['license_key'] );
			update_option( 'dictionary_pro_license', $dpro_license );
			$action = 'Removed license key.';
		} else {
			$action = 'No license key to remove.';
		}
		$response = array(
			'status' => '200',
			'message' => 'OK',
			'action_taken' => $action,
		);
		header( 'Content-Type: application/json; charset=utf-8' );
		echo json_encode( $response );
		exit;
	}

	/**
	 * Viewing Admin Columns
	 */

	function columns( $original_columns ) {
		$columns['cb'] = $original_columns['cb'];
		$columns['title'] = $original_columns['title'];
		// Dynamically add all potential columns
		$fields = apply_filters( 'dictionary_fields_order', $_GET['post_type'] );

		unset( $fields['post_content'] );
		foreach ( $fields as $uid => $field ) {
			$columns[ $uid ] = $field['label'];
		}

		$columns['shortcode'] = __( 'Shortcode', 'dictionary-pro' );
		// End custom columns
		return $columns;
	}

	function column_content( $column, $post_id ) {
		// Set default entry content to nothing
		$entry = '';
		// Global $post doesn't work with Related Posts
		$get_post = get_post( $post_id );
		$fields = apply_filters( 'dictionary_fields_order', $get_post->post_type );
		apply_filters( 'dictionary_set_taxonomy', $get_post );
		unset( $fields['post_content'] );
		foreach ( $fields as $field_uid => $field ) {
			if ( $column === $field_uid ) {
				if ( 'text' === $field['type'] ) {
					$entry = apply_filters( 'dictionary_admin_text_field', $post_id, $field_uid, $field );
				}
				if ( 'linked_post_type' === $field['type'] ) {
					$entry = apply_filters( 'dictionary_admin_linked_post_type_field', $post_id, $field_uid, $field );
				}
			}
		}
		if ( 'shortcode' === $column ) {
			$entry = "[$get_post->post_type id=$post_id]";
		}
		unset( $get_post );
		echo $entry;
	}

	function format_text_field( $post_id, $field_uid, $field_values ) {
		$transient = get_transient( $field_uid . '_admin_' . $post_id );
		if ( empty( $transient ) || false === $transient ) {

			$transient = '';
			$content = get_post_meta( $post_id, $field_uid, true );

			if ( ! empty( $content ) ) {
				$transient = $content;

				set_transient( $field_uid . '_admin_' . $post_id, $transient, HOUR_IN_SECONDS );
			}
		}
		return $transient;
	}

	function format_linked_post_type_field( $post_id, $field_uid, $field_values ) {
		// $transient = get_transient( $field_uid . '_admin_' . $post_id );
		if ( empty( $transient ) || false === $transient ) {

			$transient = '';
			$content = get_post_meta( $post_id, $field_uid, true );

			if ( ! empty( $content ) ) {

				$linked_posts = get_posts(array(
					'post_type' => $field_values['link_to'], // TODO: Validate link_to exists
					'posts_per_page' => -1,
					'post__in' => explode( ',', $content ),
				));

				$transient = '<ul class="dictionary ' . $field_uid . '_posts" style="margin-top: 0;">';
				foreach ( $linked_posts as $post ) {
					$transient .= '<li><a href="' . get_permalink( $post->ID ) . '" title="' . get_the_title( $post->ID ) . '">' . get_the_title( $post->ID ) . '</a></li>';
				}
				$transient .= '</ul>';

				wp_reset_postdata();

				set_transient( $field_uid . '_admin_' . $post_id, $transient, HOUR_IN_SECONDS );
			}
		}
		return $transient;
	}

	function reading_order( $post_type ) {
		// TODO: Support legacy options
		// $reading_order = ! empty( $legacy ) ? $legacy : array( 't', 'e', 'o', 'r' );
		// Get fields for current dictionary type
		$options = $this->dictionaries[ $post_type ]->options();
		$fields = $options['fields'];

		$base_fields = array(
			'post_content' => array(
				'uid' => 'post_content',
				'label' => 'Entry Definition',
				'type' => 'post_content',
			),
		);
		$reading_order = array(
			'post_content',
			'wpd_entry_origin',
			'wpd_entry_related',
		);
		if ( isset( $options['reading_order'] ) and ! empty( $options['reading_order'] ) ) {
			$reading_order = array_filter( $options['reading_order'] );
		}
		foreach ( $fields as $field_uid => $field_values ) {
			$base_fields[ $field_uid ] = $field_values;
		}
		$ordered = array_replace( array_flip( $reading_order ), $base_fields );

		// Removes fields that have been deleted but not re-ordered
		$display = array_filter( $ordered, function( $v ) {
			if ( is_numeric( $v ) ) { return; }
			return $v;
		});

		return $display;
	}

	/**
	 * Filter database queries
	 */

	public function alphabetical_order( $query ) {
		$sort = false;
		$dictionary_post_type = false;
		$post_type = '';
		$section_name = '';
		if ( isset( $query->query['post_type'] ) ) {
			$post_type = $query->query['post_type'];
			$dictionary_post_type = in_array( $query->query['post_type'], $this->slugs );
		}
		if ( isset( $query->query['sections'] ) ) {
			$section_name = $query->query['sections'];
		}
		if ( ! empty( $post_type ) && $dictionary_post_type  && $query->is_main_query() ) {
			$sort = true;
		}
		if ( ! empty( $section_name ) && $query->is_main_query() ) {
			$sort = true;
		}
		if ( true === $sort ) {
			$query->set( 'orderby' ,'title' );
			$query->set( 'order' ,'ASC' );
		}
	}

	public function archive_pagination( $query ) {
		foreach ( $this->dictionaries as $dictionary ) {
			$options = $dictionary->options();
			$no_paginate = ( isset( $options['archive_pagination'] ) && true == $options['archive_pagination'] );
			if ( $no_paginate && $query->is_tax( array( 'sections', 'types' ) ) && $query->is_main_query() ) {
				$query->set( 'posts_per_page', -1 );
			}
		}
	}

	public function archive_automatic_join( $join, &$query ) {
		global $wpdb;
		if ( ! $query->is_tax() ) { return $join; }
		if ( 'dictionary' !== $query->post_type() ) { return $join; }
		$join = "LEFT JOIN $wpdb->term_relationships ON ($wpdb->posts.ID = $wpdb->term_relationships.object_id)";
		return $join;
	}

	public function archive_automatic_where( $where, &$query ) {
		global $wpdb;
		$section = $query->get( 'sections' );
		$tax_query_set = isset( $query->query['tax_query'] );
		if ( ! empty( $section ) || $tax_query_set ) {
			if ( empty( $section ) && $tax_query_set ) {
				$tax_query = array_shift( $query->query['tax_query'] );
				$section = strtolower( $tax_query['terms'] );
			}
			$section = esc_sql( $section );
			$where .= " OR ( $wpdb->posts.post_title LIKE '$section%' AND  $wpdb->posts.post_type IN ('dictionary') AND ($wpdb->posts.post_status = 'publish' ) )";
		}
		return $where;
	}

	public function entry_search( $query ) {
		$search = ($query->is_search() && $query->is_main_query());
		if ( ! $search ) { return; }
		foreach ( $this->dictionaries as $dictionary ) {
			$options = $dictionary->options();
			if ( isset( $options['search_entries_only'] ) && true == $options['search_entries_only'] ) {
				$query->set( 'post_type', $dictionary->slug() );
			}
		}
	}

	/**
	 * Filtering Save Posts
	 */

	public function autoconnect_autolinks( $post_id ) {
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) { return; }

		foreach ( $this->dictionaries as $dictionary ) {
			$dictionary_options = $dictionary->options();

			$linked_post_type = $this->check_linked_post_type(
				get_post_type( $post_id ),
				$dictionary->slug(),
			$dictionary_options['fields'] );

			if ( ! $linked_post_type ) { return; }
			if ( ! isset( $_POST['content'] ) ) { return; }

			$content = $_POST['content'];

			$args = array(
				'post_type' => $dictionary->slug(),
				'nopaging'  => true,
				'post_status' => 'publish',
			);
			$existing_entries = get_posts( $args );

			$this->do_save_autoconnect_autolinks( $post_id, $content, $existing_entries, $linked_post_type );
		}
	}

	function flush_relationships( $dictionary ) {
		$args = array(
			'post_type' => $dictionary->slug(),
			'nopaging'  => true,
			'post_status' => 'publish',
		);
		$all_entries = get_posts( $args );

		$dictionary_options = $dictionary->options();
		foreach ( $all_entries as $entry ) {
			$post_id = $entry->ID;
			$post_type_slug = get_post_type( $post_id );
			$dictionary = $this->dictionaries[ $post_type_slug ];
			$linked_post_type = $this->check_linked_post_type(
				get_post_type( $post_id ),
				$dictionary->slug(),
			$dictionary_options['fields'] );
			$the_post = get_post( $post_id );
			$the_content = $the_post->post_content;
			$this->do_save_autoconnect_autolinks( $post_id, $the_content, $all_entries, $linked_post_type );
		} // End foreach existing entries
		wp_mail( 'anne@withoutboxes.com', 'Triggered autolinking', 'Ta da' );
	}

	function do_save_autoconnect_autolinks(
		$post_id,
		$post_content,
		$all_entries,
		$linked_post_type ) {
			// Post ID is the original post whose related items we need to collect
			// Set blank array to carry over related entries
			$related_entries = array();

		foreach ( $all_entries as $entry ) {
			// Don't connect entry to itself
			if ( $post_id == $entry->ID ) { continue; }

			// Make this more readable:
			$original_id = $post_id;
			$original_title = get_the_title( $post_id );
			$original_preg_title = $this->preg_title_pattern( $original_title );
			$original_content = $post_content;

			$compared_id = $entry->ID;
			$compared_title = trim( $entry->post_title );
			$compared_preg_title = $this->preg_title_pattern( $compared_title );
			$compared_content = $entry->post_content;

			// See if the title is mentioned in other content
			preg_match( $original_preg_title, $compared_content, $search );

			// Try the opposite direction if nothing is found
			if ( empty( $search ) ) {
				preg_match( $compared_preg_title, $original_content, $search );
			}

			if ( ! empty( $search ) ) {
				$related_entries[ $compared_id ] = get_post_meta( $compared_id, $linked_post_type, true );
			}
		} // End loop to examine existing entries

		if ( ! empty( $related_entries ) ) {
			// Add this entry as being related to the others
			foreach ( $related_entries as $entry_id => $entry_rels ) {
				// $this->save_relationships( $original_id, array( $entry_id ), $entry_rels, $linked_post_type );
			}
			// Add the others to this entry
			$post_relationships = get_post_meta( $original_id, $linked_post_type, true );
			$new_ids = array_keys( $related_entries );
			$this->save_relationships( $original_id, $new_ids, $post_relationships, $linked_post_type );
		}
	}

	public function preg_title_pattern( $input ) {
		$explode = explode( ' ', trim( $input ) );
		$new_array = array();
		foreach ( $explode as $e ) {
			$new_array[] = str_replace( '/', '\/', $e );
		}
		$title = implode( '[ \t]+', $new_array );
		$pattern = "/\b($title)\b/i";
		return $pattern;
	}

	public function save_relationships( $save_to_id, $new_relationships, $relationship_id_string, $linked_post_type ) {
		$existing_relationships = explode( ',', $relationship_id_string );
		foreach ( $new_relationships as $new_id ) {
			$existing_relationships[] = $new_id;
		}
		$existing_relationships = array_filter( $existing_relationships ); // Clean empty array items
		$all_related_entries = implode( ',', array_unique( $existing_relationships ) );
		update_post_meta( $save_to_id, $linked_post_type, $all_related_entries );
	}

	function check_linked_post_type( $linked_post_type, $entry_post_type, $fields ) {
		$linked = false;
		foreach ( $fields as $field_uid => $field ) {
			if (
				'linked_post_type' === $field['type']
				&& $linked_post_type === $field['link_to'] ) {
				$linked = $field_uid;
			}
		}
		return $linked;
	}

	public function autosave_taxonomy( $post ) {

		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) { return; }

		if ( ! in_array( $post->post_type, $this->slugs ) ) { return; }

		if ( $_POST && isset( $_POST['tax_input']['sections'] ) ) {
			$sections = array_filter( $_POST['tax_input']['sections'] );
		} else {
			$sections = get_transient( 'sections_for_entry-' . $post->ID );
			if ( empty( $sections ) ) {
				$sections = wp_get_post_terms( $post->ID, 'sections' );
				set_transient( 'sections_for_entry-' . $post->ID, $sections, DAY_IN_SECONDS );
			}
		}
		if ( empty( $sections ) ) {
			// Set taxonomy where none is set
			$stop_words = array( 'a', 'an', 'the' );
			$title = trim( str_replace( $stop_words, '', get_the_title( $post->ID ) ) );
			$first_letter = substr( $title, 0, 1 );
			$section = strtolower( $first_letter );
			$term = term_exists( $section, 'sections' );
			if ( null === $term || 0 === $term ) {
				$term = wp_insert_term( $section, 'sections' );
			}
			$term_id = $term['term_id'];
			$set_terms = wp_set_post_terms( $post->ID, $term_id, 'sections' );

			if ( $set_terms ) {
				// Reset the transient
				set_transient( 'sections_for_entry-' . $post->ID, $set_terms, DAY_IN_SECONDS );
			}
		}
		return $sections;
	}

	/**
	 * Documentation
	 */

	public function load_documentation() {
		$screen = get_current_screen();
		if ( ! in_array( $screen->post_type, $this->slugs ) ) { return; }
		$screen->add_help_tab(
			array(
		        'id'	=> 'dictionary-pro-101',
		        'title'	=> __( 'Dictionary Pro 101' ),
		        'content'	=> $this->load_doc( 'basic' ),
	    	)
		);
		$screen->add_help_tab(
			array(
		        'id'	=> 'dictionary-pro-entries',
		        'title'	=> __( 'Creating Entries' ),
		        'content'	=> $this->load_doc( 'entries' ),
	    	)
		);
		$screen->add_help_tab(
			array(
		        'id'	=> 'dictionary-pro-shortcodes',
		        'title'	=> __( 'Shortcodes' ),
		        'content'	=> $this->load_doc( 'shortcodes' ),
	    	)
		);
		$screen->add_help_tab(
			array(
		        'id'	=> 'dictionary-pro-bugs',
		        'title'	=> __( 'Found a bug?' ),
		        'content'	=> $this->load_doc( 'bugs' ),
	    	)
		);
		$screen->add_help_tab(
			array(
		        'id'	=> 'dictionary-pro-support',
		        'title'	=> __( 'Help &amp; Support' ),
		        'content'	=> $this->load_doc( 'support' ),
	    	)
		);
		$screen->set_help_sidebar( $this->load_doc( 'sidebar' ) );
	}

	public function load_doc( $which ) {
		global $version, $slug;
		$version = $this->version;
		$file = WPD_DIRECTORY . "/admin/partials/help-$which.php";
		if ( file_exists( $file ) ) {
			// return file_get_contents( $file );
			ob_start();
			include $file;
			return ob_get_clean();
		}
		return;
	}

	/**
	 * Per Dictionary Settings
	 */

	function register_single_settings_fields() {
		foreach ( $this->dictionaries as $dictionary ) {
			foreach ( $dictionary->settings()->sections() as $section_uid => $section_values ) {
	 			$dictionary->settings()->new_section( $section_uid, $section_values['label'], $dictionary->slug() . '-' . $section_values['tab'] );
	 		}
		}
	}

	public function register_single_settings_page() {
		foreach ( $this->dictionaries as $dictionary ) {
			add_submenu_page(
				$dictionary->settings()->url(),
				$dictionary->settings()->page_title(),
				$dictionary->settings()->menu_title(),
				'manage_options',
				'settings',
			array( $this, 'single_settings_display' ) );
		}
	}

	function single_settings_display() {
		global $dictionary;
		$post_type = $_GET['post_type'];
		$dictionary = $this->dictionaries[ $post_type ];
		include WPD_DIRECTORY . '/admin/partials/single-settings-display.php';
	}

	/**
	 * TinyMCE Settings
	 */
	function add_entry_button() {
		$screen = get_current_screen();
		if ( in_array( $screen->id, array( 'post', 'page' ) ) ) {
			echo '<button type="button" id="create-dictionary-entry" class="button" title="Create Dictionary Entry from Selected Text"><span class="dashicons dashicons-translation"></span> Create Entry</button>';
		}
	}
}
