<?php
/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://dialectpro.org/
 * @since      3.0.0
 *
 * @package    Dictionary_Pro
 * @subpackage Dictionary_Pro/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      3.0.0
 * @package    Dictionary_Pro
 * @subpackage Dictionary_Pro/includes
 * @author     Anne Dorko <hello@dialectpro.org>
 */
class Dictionary_Pro {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      Dictionary_Pro_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * The registered dictionaries for the plugin.
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      array    $dictionaries    An array of all registered dictionary objects.
	 */
	protected $dictionaries;

	/**
	 * Plugin license key
	 *
	 * @since    3.0.0
	 * @access   protected
	 * @var      string    $license    The license key for Dialect Pro memberships
	 */
	protected $license;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    3.0.0
	 */
	public function __construct() {

		$this->plugin_name = 'dictionary-pro';
		$this->version = '3.1.0';
		$this->dictionaries = array();

		$this->load_dependencies();

		$dpro_license = get_option( 'dictionary_pro_license' );
		if ( isset( $dpro_license['license_key'] ) && ! empty( $dpro_license['license_key'] ) ) {
			$this->license = $dpro_license['license_key'];
		}

		// Assuming we made it this far, go-go-gadget!
		$dictionaries = array( 'dictionary' );
		foreach ( $dictionaries as $dictionary ) {
			$this->dictionaries[ $dictionary ] = new WP_Dictionary( $dictionary, $this );
		}
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Dictionary_Pro_Loader. Orchestrates the hooks of the plugin.
	 * - Dictionary_Pro_i18n. Defines internationalization functionality.
	 * - Dictionary_Pro_Admin. Defines all hooks for the admin area.
	 * - Dictionary_Pro_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    3.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-dictionary-pro-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-dictionary-pro-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-dictionary-pro-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-dictionary-pro-public.php';

		/**
		 * The class responsible for defining new dictionaries.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-dictionary.php';

		/**
		 * The class responsible for defining additional settings class.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-dictionary-settings.php';

		/**
		 * The class responsible for creating new metaboxes on the fly.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-dictionary-metabox.php';

		/**
		 * The class responsible for filtering admin menus
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-dictionary-menu.php';

		/**
		 * The class responsible for creating new widgets.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-dictionary-widgets.php';

		/**
		 * The class handles updates.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp-dictionary-update.php';

		$this->loader = new Dictionary_Pro_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Dictionary_Pro_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    3.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Dictionary_Pro_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Dictionary_Pro_Admin( $this->get_plugin_name(), $this->get_version(), $this->get_dictionaries() );

		/**
		 * Set variable for $dictionaries
		 * Limitation of empty()
		 * http://stackoverflow.com/questions/1075534/cant-use-method-return-value-in-write-context
		 */
		$dictionaries = $this->get_dictionaries();
		if ( ! empty( $dictionaries ) ) {
			/**
			 * Check for updates
			 */
			$plugin_admin->check_updates();
		}

		/**
		 * Register Custom Filters for Developers
		 */
		$this->loader->add_filter( 'dictionary_admin_text_field', $plugin_admin, 'format_text_field', 10, 3 );
		$this->loader->add_filter( 'dictionary_admin_linked_post_type_field', $plugin_admin, 'format_linked_post_type_field', 10, 3 );
		$this->loader->add_filter( 'dictionary_set_taxonomy', $plugin_admin, 'autosave_taxonomy' );
		$this->loader->add_filter( 'dictionary_fields_order', $plugin_admin, 'reading_order' );

		/**
		 * Welcome screens for new installations
		 */
		$this->loader->add_filter( 'wp_loaded', $plugin_admin, 'redirect_to_welcome_page' );
		$this->loader->add_filter( 'admin_menu', $plugin_admin, 'register_welcome_screen' );

		/**
		 * General Plugin Functionality
		 */

		// Handle Custom Queries
		$this->loader->add_filter( 'pre_get_posts', $plugin_admin, 'alphabetical_order' );
		$this->loader->add_filter( 'pre_get_posts', $plugin_admin, 'archive_pagination' );
		$this->loader->add_filter( 'pre_get_posts', $plugin_admin, 'entry_search' );
		$this->loader->add_filter( 'posts_where', $plugin_admin, 'archive_automatic_where', 10, 2 );
		$this->loader->add_filter( 'posts_join_paged', $plugin_admin, 'archive_automatic_join', 10, 2 );

		// Automatically linking entries to related posts
		$this->loader->add_action( 'save_post', $plugin_admin, 'autoconnect_autolinks' );
		$this->loader->add_action( 'flush_entry_relationships', $plugin_admin, 'flush_relationships', 10, 1 );

		// Enable menu section filters
		$this->loader->add_filter( 'wp_nav_menu_args', $plugin_admin, 'dictionary_section_menu' );

		// Register widgets
		$this->loader->add_action( 'widgets_init', $plugin_admin, 'widgets' );

		// Per Dictionary Settings Pages
		$this->loader->add_action( 'admin_init', $plugin_admin, 'register_single_settings_fields' );
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'register_single_settings_page' );
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'register_license_page' );

		// Customized Admin Information
		$this->loader->add_action( 'admin_footer_text', $plugin_admin, 'admin_footer_text' );

		// Admin Scripts
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		if ( is_admin() ) {
			$this->loader->add_action( 'init', $plugin_admin, 'setup_tinymce_hook' );
		}

		// Support for inline documentation
		if ( get_bloginfo( 'version' ) >= '3.3' ) {
			$help_pages = array( 'post.php', 'post-new.php', 'edit.php', 'edit-tags.php' );
			foreach ( $this->dictionaries as $dictionary ) {
				$help_pages[] = $dictionary->slug() . '_page_settings';
			}
			foreach ( $help_pages as $load ) {
				$page = "load-$load";
				$this->loader->add_action( $page, $plugin_admin, 'load_documentation' );
			}
		}

		// AJAX Calls
		$this->loader->add_action(
			'wp_ajax_dictionary-pro-linked-posts',
			$plugin_admin,
			'ajax_search_linked_posts'
		);
		$this->loader->add_filter(
			'plugin_row_meta',
			$plugin_admin,
		'plugin_links', 10, 2 );
		$this->loader->add_action(
			'wp_ajax_dictionary-pro-delete-license-key',
			$plugin_admin,
			'ajax_delete_license_key'
		);

		// Dynamic Columns for Each Dictionary
		foreach ( $this->dictionaries as $dictionary ) {
			/**
			 * Dynamic Custom Columns
			 */
			$this->loader->add_filter(
				'manage_edit-' . $dictionary->slug() . '_columns',
				$plugin_admin,
			'columns' );
			$this->loader->add_filter(
				'manage_' . $dictionary->slug() . '_posts_custom_column',
				$plugin_admin,
			'column_content', 10, 2 );

		} // End Per Dictionary Loop

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Dictionary_Pro_Public( $this->get_plugin_name(), $this->get_version(), $this->get_dictionaries() );

		/**
		 * Register Custom Filters for Developers
		 */
		$this->loader->add_filter( 'dictionary_entry_link', $plugin_public, 'format_dictionary_link', 10, 4 );
		$this->loader->add_filter( 'dictionary_shortcode_title', $plugin_public, 'format_shortcode_title', 10, 2 );
		$this->loader->add_filter( 'dictionary_format_entry', $plugin_public, 'format_entry', 10, 2 );
		$this->loader->add_filter( 'dictionary_definition_field', $plugin_public, 'format_definition_field' );
		$this->loader->add_filter( 'dictionary_text_field', $plugin_public, 'format_text_field', 10, 3 );
		$this->loader->add_filter( 'dictionary_linked_post_type_field', $plugin_public, 'format_linked_post_type_field', 10, 3 );
		$this->loader->add_filter( 'dictionary_format_taxonomies', $plugin_public, 'format_dictionary_taxonomies', 10, 3 );
		$this->loader->add_filter( 'dictionary_autolinks', $plugin_public, 'autolinks', 1, 2 );

		/**
		 * General Plugin Functionality
		 */

		// Front-End Scripts
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

		// Content Filters
		$this->loader->add_action( 'the_posts', $plugin_public, 'loop' );
		$this->loader->add_action( 'the_content', $plugin_public, 'set_autolinks', 1 );

		// Enable Shortcodes for Widgets
		add_filter( 'widget_text', 'do_shortcode' );

		/**
		 * Set Shortcodes
		 * Must match functions available in public/class-dictionary-pro-public.php
		 * This provides temporary support for old shortcodes
		 */
		$shortcodes = array(
			'menu',
			'entries',
			'entry',
			'link',
		);

		// Enable options per registered dictionary
		foreach ( $this->dictionaries as $dictionary ) {
			foreach ( $shortcodes as $shortcode ) {
				add_shortcode( $dictionary->slug() . '_' . $shortcode, array( $plugin_public, $shortcode ) );
			}
			add_shortcode( $dictionary->slug(), array( $plugin_public, 'dictionary_shortcode' ) );
		} // End Per Dictionary Loop
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    3.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Dictionary_Pro_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	public function get_dictionaries() {
		return $this->dictionaries;
	}
}
