<?php
/**
 * WP_Dictionary_Metabox
 *
 * @since 3.0
 * @author Anne Dorko <anne@withoutboxes.com>
 */
class WP_Dictionary_Metabox {

	protected $dictionary; 	// Which unique Dictionary instance does this belong to?
	private $init;			// Initial args
	private $label;			// Label for field
	private $uid;			// For creating keys and variable names
	private $type;			// Set field type, i.e. text, audio, image
	private $link_to;		// For linking related post_types using this metabox

	function __construct( $dictionary, $uid = '', $args = array() ) {
		if ( ! isset( $dictionary ) || empty( $dictionary ) ) { return false; }
		$this->dictionary 	= $dictionary;

		$this->init = $args;
		$this->init['uid'] = $uid;
		$this->validate_args();

		if ( false === $this->init ) { return false; }

		// Metabox clears validations!
		extract( $this->init );
		$this->uid			= $uid;
		$this->type 		= $type;
		$this->label 		= sanitize_text_field( $label );
		if ( 'linked_post_type' === $type ) {
			$this->link_to = $link_to;
		}
		add_action( 'save_post', array( $this, 'save' ) );
	}

	public function label() {
		return $this->label;
	}
	public function id() {
		return $this->uid;
	}
	public function callback() {
		$cb = 'callback_' . $this->type;
		return $this->$cb();
	}

	/**
	 * validate_args Ensure arguments are valid, set default if not found
	 *
	 * @param  array $args Expects two keys: label, type, optional: post_type
	 * @return array       Returns validated args, or most recent failed error as string for debugging
	 */
	function validate_args() {
		$args = $this->init;
		$error = null;
		if ( empty( $args ) || null === $args ) {
			$args = array();
			$error = 'Empty: $args';
		}
		if ( ! array_key_exists( 'label', $args ) ) {
			$error = 'Key "label" not set for $args';
		}
		if ( ! array_key_exists( 'uid', $args ) ) {
			$error = 'Key "uid" not set for $args';
		}
		if ( ! array_key_exists( 'type', $args ) ) {
			$error = 'Key "type" not set for $args';
		}
		$valid_field_types = array(
			'text',
			'image',
			'audio',
			'video',
			'linked_post_type',
		);
		if ( isset( $args['type'] ) && ! in_array( $args['type'], $valid_field_types ) ) {
			$error = 'Key "type" not a valid field type in $args';
		}
		$valid_post_types = get_post_types( array(
			'public' => true,
		) );
		// TODO: Find a way to check for any registered post type
		$valid_post_types[ $this->dictionary->slug() ] = $this->dictionary->slug();
		if ( isset( $args['type'] ) && 'linked_post_type' === $args['type'] ) {
			if ( ! isset( $args['link_to'] ) || empty( $args['link_to'] ) ) {
				$error = 'Key "link_to" not present with key "linked_post_type" in $args';
			}
			// elseif ( ! in_array( $args['link_to'], $valid_post_types ) ) {
			// $error = 'Key "linked_post_type" not valid post type for $args';
			// }
		}
		if ( null !== $error ) {
			$this->init = false;
			return $error;
			exit;
		}
		if ( false !== $this->init && isset( $args['helper_text'] ) ) {
			$this->init['helper_text'] = sanitize_text_field( $args['helper_text'] );
		}
	}

	/**
	 * args Public function for init values
	 *
	 * @return array Returns array of init values, or false if init didn't pass validation
	 */
	public function args() {
		return $this->init;
	}

	public function value() {
		global $post;
		$value = get_post_meta( $post->ID, $this->uid, 1 );
		return $value;
	}

	function save() {
		// Ignore Autosave
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) { return; }

		// Verifify Post Type
		if ( empty( $_POST[ $this->dictionary->slug()  ] )
			|| ! wp_verify_nonce( $_POST[ $this->dictionary->slug()  ], plugin_basename( __FILE__ ) ) ) {
				 return; }

		// Authenticate
		global $post_id;
		if ( 'page' === $_POST['post_type'] ) {
			if ( ! current_use_can( 'edit_page', $post_id ) ) {
				return; }
		} else {
			if ( ! current_user_can( 'edit_post', $post_id ) ) {
				return; }
		}

		/*
		Save Metadata
		 */

		$data = $_POST[ $this->uid ];

		if ( 'linked_post_type' === $this->type ) {
			if ( ! empty( $data ) ) {
				$data = implode( ',', array_unique( $data ) );
			}
		}

		update_post_meta( $post_id, $this->uid, $data );
		apply_filters( 'dictionary_set_taxonomy', get_post( $post_id ) );

		// Refresh cache
		delete_transient( $this->uid . '_' . $post_id );
	}

	/**
	 * Specific Field Types
	 * 		text
	 * 		image
	 * 		audio
	 * 		video
	 * 		linked_post_type
	 * 			includes secondary post type data
	 *
	 * 	  		@param link_to string
	 */

	function callback_text() {
		$this->nonce();
		?>
		<h4 style="margin-top:0px;"><?php echo $this->label(); ?></h4>
		<textarea name="<?php echo $this->uid; ?>" class="large-text" rows="8" cols="40"><?php
		echo $this->value();
		?></textarea>
		<?php
	}
	function callback_image() {
		$this->nonce();
		echo 'Field image';
	}
	function callback_audio() {
		$this->nonce();
		echo 'Field audio';
	}
	function callback_video() {
		$this->nonce();
		echo 'Field video';
	}
	function callback_linked_post_type() {
		global $post;
		$this->nonce();

		$linked_type = get_post_type_object( $this->link_to );
		$labels = get_post_type_labels( $linked_type );

		$placeholder = sprintf( __( 'Type to %s', 'dictionary-pro' ),  $labels->search_items );
		?>
		<label class="screen-reader-text" for="suggest_<?php echo $this->uid ?>"><?php echo $placeholder; ?></label>
		<input type="text" name="suggest_<?php echo $this->link_to ?>" value="" id="suggest_<?php echo $this->uid ?>" class="regular-text" placeholder="<?php echo $placeholder; ?>">
		<p class="description"><?php echo sprintf( __( 'Create a link between any other %s and this entry.', 'dictionary-pro' ), strtolower( $labels->singular_name ) ); ?></p>
		<div id="postcustomstuff" class="related_list">
			<table id="list-table" style="text-align: left;">
			<thead>
			  <tr>
				<th class="left"><?php echo $labels->singular_name; ?> Title</th>
			  </tr>
			</thead>
			<tbody class="list-meta">
				<?php
				$transient = get_transient( $this->uid . '_admin_table_' . $post_id );
				if ( empty( $transient ) || false === $transient ) {
					$transient = '';
					$get_related_ids = get_post_meta( $post->ID, $this->uid, true );
					if ( ! empty( $get_related_ids ) ) {
						$related = get_posts(array(
							'post_type' => $this->link_to, // TODO: Validate link_to exists
							'posts_per_page' => -1,
							'post__in' => explode( ',', $get_related_ids ),
						));
						$admin_table = '';
						foreach ( $related as $related_post ) {
							$post_id = $related_post->ID;
							$edit_link = get_edit_post_link( $related_post->ID );
							$edit_title = $related_post->post_title;
							$field_uid = $this->uid;
							$row = sprintf( '<tr id="linked-item-%1$d">
							  <th><a href="%2$s">%4$s</a>
								<div class="row-actions">
									<span class="delete"><a href="#" rel="linked-item-%1$d" class="remove-linked-item">Remove</a></span>
								</div>
								<input type="hidden" name="%3$d[%1$d]" value="%1$d" id="%3$d-%1$d">
								</th>
							</tr>', $post_id, $edit_link, $field_uid, $edit_title );
							$admin_table .= $row;
						}
						wp_reset_postdata();
						set_transient( $this->uid . '_admin_table_' . $post_id, $get_related_content, HOUR_IN_SECONDS );
					}
					$transient = $admin_table;
				}
				echo $transient;
				?>
			</tbody>
			</table>
		</div>
		<?php
	}

	function nonce() {
		wp_nonce_field( plugin_basename( __FILE__ ), $this->dictionary->slug() );
	}
}
