<?php
/**
 * WP_Dictionary
 *
 * @since 3.0
 * @author Anne Dorko <anne@withoutboxes.com>
 */
class WP_Dictionary {

	protected $slug = 'dictionary';
	protected $permalink = 'dictionary';
	protected $options = '';
	protected $option_name = '';
	protected $settings = '';
	protected $taxonomies = '';
	protected $supports = array(
	  'title',
	  'editor',
	  'thumbnail',
	);
	protected $parent;

	/**
	 * Initialize the WP_Dictionary
	 *
	 * @param   string $slug   The unique identifier for this dictionary
	 */
	function __construct( $slug = '', $parent = '' ) {

		if ( ! empty( $slug ) ) {
			$this->slug = sanitize_title( $slug );
			$this->permalink = $this->slug;
		}
		$this->parent = $parent;

		$this->set_options();
		$this->set_permalink();
		$this->enable_comments();

		add_action( 'init', array( $this, 'register_posttype' ) );
		add_action( 'init', array( $this, 'register_taxonomies' ), 0 );

		$this->register_metaboxes();

		$this->settings = new WP_Dictionary_Settings( $this );
		$this->init_settings();
	}

	/**
	 * Retrieve data from Dictionary class
	 */

	public function slug() {
		return $this->slug;
	}
	public function options() {
		return $this->options;
	}
	public function option_name() {
		return $this->option_name;
	}
	public function settings() {
		return $this->settings;
	}

	/**
	 * Options:
	 * 		Set Options
	 * 		Legacy Option Support
	 * 		Default Options
	 * 		Set Permalink
	 * 		Enable Comments
	 */
	public function set_options() {
		$this->option_name = $this->slug . '_settings';

		$this->legacy_option_support();

		$options = get_option( $this->option_name );
		if ( empty( $options ) && ! isset( $options[ $this->slug ] ) ) {
			$options = $this->default_options();
		}
		if ( ! isset( $options['fields'] ) ) {
			$defaults = $this->default_options();
			$options['fields'] = $defaults['fields'];
		}

		$this->options = $options;

		return $this->options;
	}

	public function legacy_option_support() {
		$new_settings = get_option( $this->option_name );
		$old_settings = get_option( 'wpd_settings' );
		while ( empty( $new_settings ) && ! empty( $old_settings ) ) {
			add_option( $this->option_name, $old_settings );
		}
	}

	public function default_options() {
		$defaults = array(
			'enable_permalinks' => '',
			'enable_post_links' => false,
			'enable_comments' => false,
			'fields' => array(
				'wpd_entry_origin' => array(
					'label' => __( 'Origin', 'dictionary-pro' ),
					'type' 	=> 'text',
				),
				'wpd_entry_related' => array(
					'label' => __( 'Related Entries', 'dictionary-pro' ),
					'type'	=> 'linked_post_type',
					'link_to' => $this->slug(),
				),
			),
		);
		return $defaults;
	}

	public function default_labels() {
		$labels = array(
			'name' => __( 'Dictionary', 'dictionary-pro' ),
			'all_items' => __( 'All Entries', 'dictionary-pro' ),
			'singular_name' => __( 'Entry', 'dictionary-pro' ),
			'add_new' => __( 'Add New Entry', 'dictionary-pro' ),
			'add_new_item' => __( 'Add New Entry', 'dictionary-pro' ),
			'edit_item' => __( 'Edit Entry', 'dictionary-pro' ),
			'new_item' => __( 'New Entry', 'dictionary-pro' ),
			'view_item' => __( 'View Entry', 'dictionary-pro' ),
			'search_items' => __( 'Search Entries', 'dictionary-pro' ),
			'not_found' => __( 'No entries found! Please create an entry for your dictionary.', 'dictionary-pro' ),
			'not_found_in_trash' => __( 'No entries found in Trash.', 'dictionary-pro' ),
			'parent_item_colon' => __( 'Parent Entry', 'dictionary-pro' ),
			'menu_name' => __( 'Dictionary', 'dictionary-pro' ),
		);
		return $labels;
	}

	public function set_permalink() {
		$permalink = $this->options['enable_permalinks'];
		if ( isset( $permalink ) && ! empty( $permalink ) ) {
			$this->permalink = sanitize_title( $this->options['enable_permalinks'] );
		}
		return $this->permalink;
	}

	public function enable_comments() {
		if ( isset( $this->options['enable_comments'] ) && 'true' === $this->options['enable_comments'] ) {
			$this->supports[] = 'comments';
		}
	}

	/**
	 * Save things from the outside
	 */


	/**
	 * Register Custom Post Type for Dictionary
	 *
	 * @return null
	 */
	function register_posttype() {
		register_post_type( $this->slug(),
			array(
				'labels' => $this->default_labels(),
				'description' => __( 'The WordPress Dictionary post type allows you to support dictionary entries for  your WordPress installation.', 'dictionary-pro' ),
				'public' => true,
				'has_archive' => true,
				'publicly_queryable' => true,
				'menu_position' => 20,
				'menu_icon'           => 'dashicons-translation',
				'rewrite' => array(
					// Sanitize for URLs, in case something weird made it here
					'slug' => sanitize_title( $this->permalink ),
					'with_front' => false,
				),
				'supports' => $this->supports,
			)
		);
	}

	/**
	 * Register Taxonomies
	 */

	public function register_taxonomies() {
		$this->set_taxonomies();

		foreach ( $this->taxonomies as $taxonomy => $taxonomy_args ) {
			register_taxonomy( $taxonomy, $this->slug(), $taxonomy_args );
		}

		$this->set_default_taxonomies();
	}

	// Taxonomy Types
	public function set_taxonomies() {
		$taxonomies = array();
		$taxonomies['sections'] = array(
			'labels' => array(
				'name' => __( 'Entry Sections', 'dictionary-pro' ),
				'singular_name' => __( 'Section', 'dictionary-pro' ),
				'search_items' => __( 'Search Sections', 'dictionary-pro' ),
				'popular_items' => __( 'Popular Sections', 'dictionary-pro' ),
				'all_items' => __( 'All Sections', 'dictionary-pro' ),
				'edit_item' => __( 'Edit Section', 'dictionary-pro' ),
				'add_new_item' => __( 'Add New Section', 'dictionary-pro' ),
				'update_item' => __( 'Update Section', 'dictionary-pro' ),
				'new_item_name' => __( 'Add New Section', 'dictionary-pro' ),
				'add_or_remove_items' => __( 'Add or Remove Sections', 'dictionary-pro' ),
				'menu_name' => __( 'Entry Sections', 'dictionary-pro' ),
			),
			'hierarchical' => true,
			'rewrite' => array(
				'slug' => 'sections',
			),
			'defaults' => range( 'A', 'Z' ),
		);
		$taxonomies['types'] = array(
			'labels' => array(
				'name' => __( 'Entry Types', 'dictionary-pro' ),
				'singular_name' => __( 'Type', 'dictionary-pro' ),
				'search_items' => __( 'Search Types', 'dictionary-pro' ),
				'popular_items' => __( 'Popular Types', 'dictionary-pro' ),
				'all_items' => __( 'All Types', 'dictionary-pro' ),
				'edit_item' => __( 'Edit Type', 'dictionary-pro' ),
				'add_new_item' => __( 'Add New Type', 'dictionary-pro' ),
				'update_item' => __( 'Update Type', 'dictionary-pro' ),
				'new_item_name' => __( 'Add New Type', 'dictionary-pro' ),
				'add_or_remove_items' => __( 'Add or Remove Types', 'dictionary-pro' ),
				'menu_name' => __( 'Entry Types', 'dictionary-pro' ),
			),
			'hierarchical' => true,
			'rewrite' => array(
				'slug' => 'types',
			),
			'defaults' => array(
				'Verb',
				'Noun',
				'Adjective',
				'Adverb',
				'Pronoun',
				'Preposition',
				'Conjunction',
				'Interjection',
			),
		);
		$this->taxonomies = $taxonomies;
	}

	public function set_default_taxonomies() {
		foreach ( $this->taxonomies as $taxonomy => $taxonomy_args ) {
			// If the taxonomy is here but empty
			$terms = get_terms( $taxonomy );
			if ( taxonomy_exists( $taxonomy ) && empty( $terms ) ) {
				// NOTE: This is recorded as bad practice, look for alternatives
				flush_rewrite_rules( false );

				foreach ( $taxonomy_args['defaults'] as $default ) {
					wp_set_object_terms( null, $default, $taxonomy, false );
				}
			}
		}
	}

	/**
	 * Individual Entry Metaboxes
	 */

	function register_metaboxes() {
		$fields = $this->order_fields( $this->options['fields'], $this->options );
		unset( $fields['post_content'] );
		$this->metaboxes = array();
		foreach ( $fields as $field_name => $field_args ) {
			if ( is_array( $field_args ) ) {
				$this->metaboxes[ $field_name ] = new WP_Dictionary_Metabox( $this, $field_name, $field_args );
			}
		}
		add_action( 'add_meta_boxes', array( $this, 'register_metabox' ) );
	}

	function order_fields( $fields, $options ) {
		// Get fields for current dictionary type
		$base_fields = array(
			'post_content' => array(
				'uid' => 'post_content',
				'label' => 'Entry Definition',
				'type' => 'post_content',
			),
		);
		$reading_order = array(
			'post_content',
			'wpd_entry_origin',
			'wpd_entry_related',
		);
		if ( isset( $options['reading_order'] ) and ! empty( $options['reading_order'] ) ) {
			$reading_order = $options['reading_order'];
		}
		foreach ( $fields as $field_uid => $field_values ) {
			$base_fields[ $field_uid ] = $field_values;
		}
		$ordered = array_replace( array_flip( $reading_order ), $base_fields );
		return $ordered;
	}

	function register_metabox() {
		$callback_function = 'build_metabox';
		$default_labels = $this->default_labels();
		add_meta_box(
			$this->slug() . '-postmeta',
			'Additional ' . $default_labels['singular_name'] . ' Data',
			array( $this, $callback_function ),
			$this->slug(),
			'normal',
			'high'
		);
	}
	function build_metabox() {
		global $dictionary;
		$dictionary = $this;
		include WPD_DIRECTORY . '/admin/partials/entry-metabox.php';
	}


	/**
	 * Individual Dictionary Settings
	 */

	function init_settings() {
		include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		$settings = $this->settings();

		$settings->add_section(
			'general',
			'wpd_license_key',
			__( 'Dialect Pro Membership', 'dictionary-pro' ),
			'<p>' . __( 'Dictionary Pro is distributed with membership at Dialect Pro.', 'dictionary-pro' ) . ' <a href="https://dialectpro.org/account/" target="_blank">' . __( 'Find your license key here.', 'dictionary-pro' ) . '</a></p>'
		);
			$options = get_option( 'dictionary_pro_license' );
			$disable_license_field = '';
			$license_key = '';
			$url = get_home_url();
			$disallowed = array( 'http://', 'https://' );
		foreach ( $disallowed as $d ) {
			if ( strpos( $url, $d ) === 0 ) {
				$url = str_replace( $d, '', $url );
			}
		}
			$blog_url = $url;
		if ( isset( $options['license_key'] ) && ! empty( $options['license_key'] ) ) {
			$license_key = $options['license_key'];
			$disable_license_field = 'disabled';
			$license_data = '<input type="text" class="regular-text" placeholder="DIALECT_XXXXXXXXXXX" value="' . $license_key . '" ' . $disable_license_field . ' />
			<p id="license-active" style="font-size: .9em;">' . __( 'Your license is currently active.', 'dictionary-pro' ) . ' <a href="#" id="deregister" style="color: #D54E21;">' . __( 'Click to deregister this site.', 'dictionary-pro' ) . '</a></p>
			<input type="hidden" id="dpro_domain" value="' . $blog_url . '" />
			<input type="hidden" id="dpro_email" value="' . $options['license_email'] . '" />
			<input type="hidden" id="dpro_server" value="' . DICTIONARY_PRO_API_SERVER . '" />
			<input type="hidden" id="dpro_ref" value="' . DICTIONARY_PRO_REFERENCE . '" />
			<input type="hidden" id="dpro_password" value="' . DICTIONARY_PRO_SECRET_KEY . '" />';
		} else {
			$activate_url = admin_url( 'edit.php?post_type=' . $this->slug() . '&page=activation' );
			$license_data = '<p><a href="' . $activate_url . '" class="button">' . __( 'Activate for Updates' ) . '</a></p>';
		}
		$settings->add_field(
			'wpd_license_key',
			'license_key',
			__( 'License', 'dictionary-pro' ),
			$license_data
		);
			$settings->add_section(
				'general',
				'wpd_post_settings',
				__( 'Site Settings', 'dictionary-pro' ),
				'<p>' . __( 'Manage the way your dictionary interacts with the rest of your site.', 'dictionary-pro' ) . '</p>'
			);
			$settings->add_field(
				'wpd_post_settings',
				'enable_post_links',
				__( 'Post Linking', 'dictionary-pro' ),
				'<label><input type="checkbox" value="true" /> '
					. __( 'Automatically link to dictionary entries in posts', 'dictionary-pro' ) . '</label>'
			);
			$settings->add_field(
				'wpd_post_settings',
				'disable_empty_menus',
				__( 'Section Menus', 'dictionary-pro' ),
				'<label><input type="checkbox" value="true" /> '
					. __( 'Make sections without any entries unclickable in menus', 'dictionary-pro' ) . '</label>'
			);
			$settings->add_field(
				'wpd_post_settings',
				'search_entries_only',
				__( 'Search', 'dictionary-pro' ),
				'<label><input type="checkbox" value="true" /> '
					. __( 'Restrict all site search to dictionary entries only', 'dictionary-pro' ) . '</label>'
			);
			// Simple Tooltips plugin support
		if ( is_plugin_active( 'simple-tooltips/index.php' ) ) {
			$settings->add_field(
				'wpd_post_settings',
				'enable_simple_tooltips',
				__( 'Simple Tooltips' ),
				'<label><input type="checkbox" value="true" /> '
				. __( 'Show definition in a tooltip for autolinked dictionary entries', 'dictionary-pro' ) . '</label>'
			);
		} else {
			$settings->add_field(
				'wpd_post_settings',
				'enable_simple_tooltips',
				__( 'Simple Tooltips' ),
				'<label><input type="checkbox" value="" disabled /> <a href="' . get_admin_url() . 'plugin-install.php?s=Simple+Tooltips&tab=search&type=term">'
				. __( 'Install Simple Tooltips for more options', 'dictionary-pro' ) . '</a></p></label>'
			);
		}

			$settings->add_section(
				'general',
				'wpd_per_entry',
				__( 'Entry Settings', 'dictionary-pro' ),
				'<p>' . __( 'Manage the functionality of your dictionary entries on the site.', 'dictionary-pro' ) . '</p>'
			);
			$settings->add_field(
				'wpd_per_entry',
				'enable_comments',
				__( 'Discussion', 'dictionary-pro' ),
				'<label><input type="checkbox" value="true" /> '
					. __( 'Enable comments for dictionary entries', 'dictionary-pro' ) . '</label>'
			);

			$settings->add_field(
				'wpd_per_entry',
				'enable_permalinks',
				__( 'Permalink Base', 'dictionary-pro' ),
				'<input type="text" class="regular-text" placeholder="dictionary" value="" />'
			);

			$settings->add_section(
				'appearance',
				'wpd_appearance',
				__( 'Styles and Appearance', 'dictionary-pro' ),
				'<p>' . __( 'Change how the dictionary appears to your readers.', 'dictionary-pro' ) . '</p>'
			);

			$settings->add_field(
				'wpd_appearance',
				'default_styles',
				__( 'Enable CSS', 'dictionary-pro' ),
				'<label><input type="checkbox" value="true" /> '
					. __( 'Use built-in dictionary styles alongside your theme', 'dictionary-pro' ) . '</label>'
			);

			$settings->add_field(
				'wpd_appearance',
				'titles_only',
				__( 'Titles Only', 'dictionary-pro' ),
				'<label><input type="checkbox" value="true" /> '
					. __( 'Show entry titles without definitions on archive pages', 'dictionary-pro' ) . '</label>'
			);

			$settings->add_field(
				'wpd_appearance',
				'display_taxonomies',
				__( 'Display Taxonomies', 'dictionary-pro' ),
				'<label><input type="checkbox" value="true" /> '
					. __( 'Show taxonomies (sections and types) with full entry content', 'dictionary-pro' ) . '</label>'
			);

			$settings->add_field(
				'wpd_appearance',
				'archive_pagination',
				__( 'Section Pagination', 'dictionary-pro' ),
				'<label><input type="checkbox" value="true" /> '
					. __( 'Display all entries under each section archive without pagination', 'dictionary-pro' ) . '</label>'
			);

			$settings->add_field(
				'wpd_appearance',
				'field_order',
				__( 'Display Order', 'dictionary-pro' ),
				function( $dictionary ) {
					$string = '<p class="description">' . __( 'You can click and drag to reorder the way your dictionary entries display on the site.', 'dictionary-pro' ) . '</p>
			    <h3>' . __( 'Entry Title', 'dictionary-pro' ) . '</h3>
				<ul id="sortable">
				';

					$fields = apply_filters( 'dictionary_fields_order', $dictionary->slug() );

					foreach ( $fields as $field_uid => $field ) {
						$string .= '<li class="sort ui-state-default">
						<p><strong>' . $field['label'] . '</strong></p>
						<input type="hidden" name="' . $dictionary->option_name() . '[reading_order][]" value="' . $field_uid . '" id="reading_order_' . $field_uid . '"></li>';
					}

					$string .= '</ul>';

					return $string;
				}
			);

			$settings->add_section(
				'advanced',
				'wpd_custom_fields',
				__( 'Advanced Customization', 'dictionary-pro' ),
				'<p>' . __( 'Manage the way your dictionary works. Proceed with caution! If you are working with an existing dictionary, please make a backup before using the following features.', 'dictionary-pro' ) . '</p>'
			);

			$settings->add_field(
				'wpd_custom_fields',
				'flush',
				__( 'Flush Relationships', 'dictionary-pro' ),
				function ( $dictionary ) {
					if ( isset( $_REQUEST['flush-links'] ) && true == $_REQUEST['flush-links'] ) {
						// Schedule job to process dictionaries
						$schedule = wp_schedule_single_event( time(), 'flush_entry_relationships', $dictionary->slug() );
						// do_action( 'flush_entry_relationships', $dictionary );
						// $this->parent->flush_entry_relationships( $dictionary );
						echo '<p>' . __( 'Linked fields are automatically building relationships now.', 'dictionary-pro' ) . '</p>';
					} else {
						echo '<p><a href="' . $_SERVER['REQUEST_URI'] . '&flush-links=true" class="button">' . __( 'Regenerate Linked Relationships', 'dictionary-pro' ) . '</a></p>
						<p class="description">' . __( 'This will regenerate the automatically managed relationships between entries and the rest of your site. If you have a large database, this could take a while. Useful if you recently added a new linked post type to an existing dictionary.', 'dictionary-pro' ) . '</p>';
					}
				}
			);

			$settings->add_field(
				'wpd_custom_fields',
				'fields',
				__( 'Entry Information', 'dictionary-pro' ),
				function( $dictionary ) {
					/*
					Types of fields: Text, Image, Audio, Video, Linked Entries (examples: Synonyms, Antonyms)
		            Add field: Popup with text input to edit name, dropdown to add a type (default: text)
					Edit field: Popup with text input to edit name, dropdown to modify type
					Delete field: Confirm box with warning where you must type "DELETE" similar to Mailchimp
					*/
					$field_types = array(
					   'text' => 'Text',
					// 'image' => 'Image Source',
					// 'audio' => 'Audio Source',
					// 'video' => 'Video Source',
					   'linked_post_type' => 'Related Post Type',
				    );
					$string = '<label><strong class="screen-reader-text">' . __( 'Add Field:', 'dictionary-pro' ) . '</strong> <input type="text" name="new_field" value="" id="new_field" class="regular-text" placeholder="Field Name"></label>';
					$string .= '<label><strong class="screen-reader-text">' . __( 'Select Type:', 'dictionary-pro' ) . '</strong>
				<select id="choose_field_type" name="choose_field_type">
				<option value="">– Select Field Type –</option>';
					foreach ( $field_types as $type => $label ) {
						$string .= '<option value=' . $type . '>' . $label . '</option>';
					}
					$string .= '</select>
				</label>';

					$string .= '<select id="choose_link_to" name="choose_link_to" class="hidden">';
					foreach ( get_post_types( array( 'public' => true ) ) as $post_types => $values ) {
						if ( 'attachment' !== $values ) {
							$string .= '<option value="' . $values . '">' . ucwords( $post_types ) . '</option>';
						}
					}
					$string .= '</select>';

					$string .= '<input type="button" value="Add Field" class="button" id="wpd_add_field" />';

					$string .= '<div id="postcustomstuff" class="related_list">
							<table id="list-table" style="text-align: left;">
							<thead>
							<tr>
							  <th class="left">' . __( 'Field', 'dictionary-pro' ) . '</th>
							</tr>
							</thead>
							<tbody class="list-meta">';
					$permanent = array(
					'post_title' => array(
						'uid' => 'post_title',
						'label' => 'Entry Title',
						'type'	=> 'text',
					),
					'post_content' => array(
						'uid' => 'post_content',
						'label' => 'Entry Definition',
						'type'	=> 'WYSIWYG text',
					),
					);
					foreach ( $permanent as $field => $values ) {
						$string .= '<tr id="dictionary-field-' . $values['uid'] . '" class="">
			            <th>
			                <div class="field-info">
			                    <span class="name">' . $values['label'] . '</span> <span class="type">' . $values['type'] . '</span>
			                </div>
			                <div class="row-actions">
			                    <span>Permanent</span>
			                </div>
			            </th>
			        </tr>';
					}
					$options = $dictionary->options();
					$fields = $options['fields'];
					foreach ( $fields as $field => $values ) {
						$field = sanitize_title( $field );
						$string .= '<tr id="dictionary-field-' . $field . '" class="set-dictionary-field">
		                <th>
		                    <div class="field-info">
		                        <span class="name">' . $values['label'] . '</span> <span class="type">' . $values['type'] . '</span>
		                    </div>
		                    <div class="row-actions">
		                        <span class="edit"><a href="#TB_inline?width=300&height=170&inlineId=thickbox-' . $field . '" rel="dictionary-field-' . $field . '" class="edit-field thickbox">' . __( 'Edit Field', 'dictionary-pro' ) . '</a> | </span>
		                        <span class="delete"><a href="#delete?field=' . $field . '" rel="dictionary-field-' . $field . '" class="remove-field">' . __( 'Delete Field', 'dictionary-pro' ) . '</a></span>
		                    </div>
							<input type="hidden" name="' . $dictionary->option_name() . '[fields][' . $field . '][type]" value="' . $values['type'] . '" />
							<input type="hidden" name="' . $dictionary->option_name() . '[fields][' . $field . '][label]" value="' . $values['label'] . '" />';
						if ( 'linked_post_type' === $values['type'] ) {
							$string .= '<input type="hidden" name="' . $dictionary->option_name() . '[fields][' . $field . '][link_to]" value="' . $values['link_to'] . '" />';
						}
						$string .= '<div id="thickbox-' . $field . '" style="display:none;">
								     <h3>Editing ' . $values['label'] . '</h3>
									 <p><label><strong>Change label:</strong> <input type="text" value="' . $values['label'] . '" id="change-dictionary-field-' . $field . '" /></label>
									 <span class="message dictionary-field-' . $field . '"></span></p>
									 <a href="#" rel="dictionary-field-' . $field . '" class="button button-primary update-field">Update</a>
								</div>';
						$string .= '</th>
		            </tr>';
					}
					$string .= '</tbody></table>';
					return $string;
				}
			);
	}
}
