<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://dialectpro.org/
 * @since      3.0.0
 *
 * @package    Dictionary_Pro
 * @subpackage Dictionary_Pro/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Dictionary_Pro
 * @subpackage Dictionary_Pro/public
 * @author     Anne Dorko <hello@dialectpro.org>
 */
class Dictionary_Pro_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The version of this plugin.
	 *
	 * @since    3.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $dictionaries;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    3.0.0
	 * @param      string $plugin_name       The name of the plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $dictionaries ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->dictionaries = $dictionaries;
		$this->slugs = array();
		foreach ( $dictionaries as $dictionary ) {
			$this->slugs[] = $dictionary->slug();
		}

	}

	public function loop( $posts ) {

		if ( empty( $posts ) ) { return $posts; }

		foreach ( $posts as $post ) {
			if ( in_array( $post->post_type, $this->slugs ) ) {
				setup_postdata( $post );
				$entry = apply_filters( 'dictionary_format_entry', $post );
				if ( ! is_archive() ) {
					$post->post_content = $entry;
				} else {
					$options = $this->dictionaries[ $post->post_type ]->options();
					if ( isset( $options['titles_only'] ) && true == $options['titles_only'] ) {
						$post->post_content = '';
					} else {
						$post->post_content = $entry;
					}
				}
			}
		}
		return $posts;
	}

	public function excerpt( $excerpt ) {
		global $post;
		// Only use this for dictionary types
		if ( in_array( $post->post_type, $this->slugs ) ) {
			// Get the content and apply filters
			$get_entry = apply_filters( 'the_content', $post->post_content );
			$clean_entry = str_replace( ']]>', ']]&gt;', $get_entry );
			$excerpt = $clean_entry;
		}
		return $excerpt;
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    3.0.0
	 */
	public function enqueue_styles() {

		$dictionaries = $this->dictionaries;
		$options = array_shift( $dictionaries )->options();

		if ( isset( $options['default_styles'] ) && true == $options['default_styles'] ) {
			wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/dictionary-pro-public.css', array(), $this->version, 'all' );
		}

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    3.0.0
	 */
	public function enqueue_scripts() {

		// wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/plugin-name-public.js', array( 'jquery' ), $this->version, false );
	}

	function reading_order( $post_type ) {
		// TODO: Support legacy options
		// $reading_order = ! empty( $legacy ) ? $legacy : array( 't', 'e', 'o', 'r' );
		// Get fields for current dictionary type
		$options = $this->dictionaries[ $post_type ]->options();
		$fields = $options['fields'];

		$base_fields = array(
			'post_content' => array(
				'uid' => 'post_content',
				'label' => 'Entry Definition',
				'type' => 'post_content',
			),
		);
		$reading_order = array(
			'post_content',
			'wpd_entry_origin',
			'wpd_entry_related',
		);
		if ( isset( $options['reading_order'] ) and ! empty( $options['reading_order'] ) ) {
			$reading_order = array_filter( $options['reading_order'] );
		}
		foreach ( $fields as $field_uid => $field_values ) {
			$base_fields[ $field_uid ] = $field_values;
		}
		$ordered = array_replace( array_flip( $reading_order ), $base_fields );

		// Removes fields that have been deleted but not re-ordered
		$display = array_filter( $ordered, function( $v ) {
			if ( is_numeric( $v ) ) { return; }
			return $v;
		});

		return $display;
	}

	/**
	 * format fields front facing functions
	 *
	 * @param  integer $post_id      The post ID being filtered
	 * @param  integer $field_uid    The custom Dictionary Field UID
	 * @param  array   $field_values The values saved for this Dictionary Field
	 * @return string                The final result of saved field data
	 */

	function format_shortcode_title( $post_title, $before = '<h2>', $after = '</h2>' ) {
		return $before . $post_title . $after;
	}

	function format_definition_field( $post_content ) {
		return wpautop( convert_chars( wptexturize( $post_content ) ) );
	}

	function format_text_field( $post_id, $field_uid, $field_values ) {
		$transient = get_transient( $field_uid . '_' . $post_id );
		if ( empty( $transient ) || false === $transient ) {
			$transient = '';
			$content = get_post_meta( $post_id, $field_uid, true );
			if ( ! empty( $content ) ) {
				$transient = '<div class="entry-meta ' . $field_uid . '">
                  <h3 class="entry-meta-heading">' . $field_values['label'] . ':</h3>
                  <p class="entry-meta-content">' . $content . '</p>
                  </div>';

				set_transient( $field_uid . '_' . $post_id, $transient, HOUR_IN_SECONDS );
			}
		}
		return $transient;
	}

	function format_linked_post_type_field( $post_id, $field_uid, $field_values ) {
		$transient = get_transient( $field_uid . '_' . $post_id );
		if ( empty( $transient ) || false === $transient ) {
			$transient = '';
			$content = get_post_meta( $post_id, $field_uid, true );
			if ( ! empty( $content ) ) {

				$linked_posts = get_posts(array(
					'post_type' => $field_values['link_to'], // TODO: Validate link_to exists
					'posts_per_page' => -1,
					'post__in' => explode( ',', $content ),
				));

				$transient = '<div class="entry-meta linked-posts ' . $field_uid . '">
                  <h3 class="entry-meta-heading">' . $field_values['label'] . ':</h3>
                  <ul class="entry-meta-content">';
				foreach ( $linked_posts as $post ) {
					$transient .= '<li><a href="' . get_permalink( $post->ID ) . '" title="' . get_the_title( $post->ID ) . '">' . get_the_title( $post->ID ) . '</a></li>';
				}

				$transient .= '</ul></div>';

				wp_reset_postdata();

				set_transient( $field_uid . '_' . $post_id, $transient, HOUR_IN_SECONDS );
			}
		}
		return $transient;
	}

	function format_dictionary_taxonomies( $post_id, $taxonomy, $terms ) {
		$taxonomy = '<div class="entry-meta ' . $taxonomy->name . '"><h3>' . $taxonomy->label . ':</h3>';
		$taxonomy .= '<ul class="dictionary ' . $taxonomy->name . '_taxonomy" style="margin-top: 0;">';
		foreach ( $terms as $term ) {
			$taxonomy .= '<li><a href="' . get_term_link( $term->term_id ) . '" title="' . $term->name . '">' . $term->name . '</a></li>';
		}
		$taxonomy .= '</ul></div>';
		return $taxonomy;
	}

	public function format_dictionary_link( $entry, $dictionary_slug = 'dictionary', $before_link = '', $after_link = '' ) {
		$title = get_the_title( $entry->ID );
		$post_link = get_post_permalink( $entry->ID );

		$link = '<a href="' . $post_link . '" title="' . __( 'Define ', 'dictionary-pro' ) . $entry->post_title . '" class="' . $dictionary_slug . '-link entry_' . $entry->post_name . '" id="entry-' . $entry->ID . '">$1</a>';

		$options = $this->dictionaries[ $dictionary_slug ]->options();
		/**
		 * Support for Simple Tooltips
		 * https://wordpress.org/plugins/simple-tooltips/
		 */
		if ( isset( $options['enable_simple_tooltips'] ) && true == $options['enable_simple_tooltips'] ) {
			if ( ! is_admin() ) {
				include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
			}
			if ( is_plugin_active( 'simple-tooltips/index.php' ) ) {
				$get_entry = apply_filters( 'dictionary_definition_field', $entry->post_content );
				$enable_tooltip = '[simple_tooltip content="' . $get_entry . '"]' . $link . '[/simple_tooltip]';
				$link = do_shortcode( $enable_tooltip );
			}
		}
		return $before_link . $link . $after_link;
	}

	/**
	 * format_entry – Display the full dictionary entry
	 *
	 * @param  [type] $post [description]
	 * @return [type]       [description]
	 */
	public function format_entry( $post, $shortcode = false ) {
		$fields = apply_filters( 'dictionary_fields_order', $post->post_type );
		$taxonomies = get_object_taxonomies( $post, 'objects' );
		$options = $this->dictionaries[ $post->post_type ]->options();

		$user_friendly_id = sanitize_title( get_the_title( $post->ID ) );

		$entry = '<div id="entry-' . $user_friendly_id . '" class="dictionary-pro entry entry-' . $post->ID . ' entry-' . $user_friendly_id . '"><!-- Get documentation for the Dictionary Pro WordPress plugin @ https://dialectpro.org/ -->';
		if ( $shortcode ) {
			$entry .= apply_filters( 'dictionary_shortcode_title', get_the_title( $post->ID ) );
		}
		foreach ( $fields as $field_uid => $field ) {
			if ( 'post_content' === $field['type'] ) {
				$entry .= apply_filters( 'dictionary_definition_field', $post->post_content );
			}
			if ( 'text' === $field['type'] ) {
				$entry .= apply_filters( 'dictionary_text_field', $post->ID, $field_uid, $field );
			}
			if ( 'linked_post_type' === $field['type'] ) {
				$entry .= apply_filters( 'dictionary_linked_post_type_field', $post->ID, $field_uid, $field );
			}
		}
		if ( isset( $options['display_taxonomies'] ) && true == $options['display_taxonomies'] ) {
			foreach ( $taxonomies as $taxonomy ) {
				$taxes = wp_get_post_terms( $post->ID, $taxonomy->name );
				$entry .= apply_filters( 'dictionary_format_taxonomies', $post->ID, $taxonomy, $taxes );
			}
		}
		$entry .= '</div>';
		wp_reset_postdata();
		return $entry;
	}

	public function set_autolinks( $content ) {
		foreach ( $this->dictionaries as $dictionary ) {
			$dictionary_options = $dictionary->options();

			if (
				isset( $dictionary_options['enable_post_links'] ) and
				true == $dictionary_options['enable_post_links'] ) {

				$content = apply_filters( 'dictionary_autolinks', $content, $dictionary );

			}
		}
		return $content;
	}

	public function autolinks( $content, $dictionary ) {
		if ( in_array( get_post_type(), array( 'post' ) ) ) {
			global $post;
			// Look for existing cached automagic keys
			$entries = get_transient( $dictionary->slug() . '_entries_automagic_keys_' . $post->ID );
			$entries = null;
			$entry_links = array();
			if ( ! $entries ) {
				// Get all dictionary terms...
				$args = array(
					'post_type' => $dictionary->slug(),
					'nopaging'  => true,
					'post_status' => 'publish',
				);
				$existing_entries = get_posts( $args );
				foreach ( $existing_entries as $entry ) {
					// {1} - number of substitutions to make
					// \b - exact match only (i.e. not part of another word)
					// /i - case insensitive
					// (?!([A-Za-z0-9 _.,!"'/$]+)<\/a>) -- Checks that there is not an open end to an existing link somewhere ahead
					$title = str_replace( ' ', '[ \t]+', preg_quote( trim( $entry->post_title ), '/' ) );
					$key = '/\b(' . $title . ')\b(?!([a-zA-Z0-9-\/.]+"))(?!">)(?![ a-zA-Z0-9_="\/ ]+">(.*)<\/a>)(?!([A-Za-z0-9-\/ .!?]+)*<\/a>)/i';
					preg_match( $key, $content, $search );
					if ( empty( $search ) ) { continue; } // Only include entries actually in the text
					$entry_links[ $key ] = apply_filters( 'dictionary_entry_link', $entry, $dictionary->slug() );
					$key = null;
				}
				wp_reset_query(); // Necessary or else loops get confused.
				set_transient( $dictionary->slug() . '_entries_automagic_keys_' . $post->ID, $entry_links, HOUR_IN_SECONDS );
			}
			$do_links = preg_replace( array_keys( $entry_links ), array_values( $entry_links ), $content, 1 );
			if ( null !== $do_links ) {
				// Fix for error where preg_replace returns null
				$content = $do_links;
			}
		}
		return $content;
	}



	/**
	 * Shortcode Functions
	 */

	function dictionary_shortcode( $atts ) {

		if ( in_array( 'menu', $atts ) ) {
			return $this->menu( $atts );
			exit;
		}
		if ( in_array( 'entries', $atts ) ) {
			return $this->entries( $atts );
			exit;
		}

		if ( in_array( 'link', $atts ) ) {
			return $this->link( $atts );
			exit;
		}

		// Default to entry if an ID is set and nothing else triggered
		if ( isset( $atts['id'] ) ) {
			return $this->entry( $atts );
			exit;
		}
	}

	function menu( $atts ) {
			// Ignore all attributes right now, only pulling Sections
			$sections = get_terms( array(
				'taxonomy' => 'sections',
				'hide_empty' => false,
			) );

			// Generate a list of Sections
			$menu = '<ul class="wp_menu dictionary-pro shortcode">';
		foreach ( $sections as $section ) {
			$menu .= '<li><a href="' . get_term_link( $section ) . '">' . $section->name . '</a></li>';
		}
			$menu .= '</ul>';

			// Return the menu
			return $menu;
	}

	/*
     Returns an array of entries based on Section or Type

        Usage:  [dictionary_entries]
                 - Defaults to all dictionary post types

                 [dictionary_entries random="5"]
                 - Returns array of random dictionary post types limited by the number provided

                 [dictionary_entries section="a"]
                 - Returns array of dictionary post types that start with "a" or have the Section "a"

                 [dictionary_entries type="noun"]
                 - Returns array of dictionary post types that have the Type "noun"

                 [dictionary_entries section="a" type="noun"]
                 - Returns array of dictionary post types that start with "a" or have the Section "a"
                 - Filters entries to only ones that have the Type "noun"
     */
	function entries( $atts ) {
		extract( shortcode_atts( array(
			'section' 	=> 'none',
			'type' 		=> 'none',
			'random' 	=> 'none',
			'display' 	=> 'titles',
		), $atts) );

		// Baseline Search Query
		$search = array(
			'post_type' => array_shift( $this->slugs ), // TODO: Make this inclusive of all slugs
			'orderby' 	=> 'title',
			'order' 	=> 'ASC',
		);

		if ( 'none' !== $section ) {
			$search['tax_query'][] = array(
				'taxonomy' 	=> 'sections',
				'field' 	=> 'name',
				'terms' 	=> $section,
			);
		}

		if ( 'none' !== $type ) {
			$search['tax_query'][] = array(
				'taxonomy' => 'types',
				'field' => 'slug',
				'terms' => $type,
			);
		}

		if ( 'none' !== $random ) {
			$search['orderby'] = 'rand';
			// $search['order']   = 'RAND()';
			$search['numberposts'] = $random;
		}

		$wpd_entries = get_posts( $search );

		// Make sure everything is unique
		if ( ! empty( $wpd_entries ) and ! empty( $wpd_auto_sections ) ) {
			$merged = array_merge( $wpd_entries, $wpd_auto_sections );
			$postids = array();
			foreach ( $merged as $item ) {
				$postids[] = $item->ID;
			}
			$uniqueposts = array_unique( $postids );
			$entries = get_posts( array(
				'post__in' => $uniqueposts,
				'post_type' => 'dictionary',
				'post_status' => 'publish',
			) );
		} elseif ( ! empty( $wpd_entries ) and empty( $wpd_auto_sections ) ) {
			$entries = $wpd_entries;
		} elseif ( empty( $wpd_entries ) and ! empty( $wpd_auto_sections ) ) {
			$entries = $wpd_auto_sections;
		}
		// Get all of the entries, formatted
		$output = '';
		if ( ! empty( $entries ) ) {
			foreach ( $entries as $entry ) {
				// Format entries
				$output .= apply_filters( 'dictionary_format_entry', $entry, true );
			}
		}
		// Return formatted entries
		return $output;
	}


	function entry( $atts ) {
		extract( shortcode_atts( array(
			'id'      => 'default',
			'slug'    => 'default',
			'display' => 'default',
		), $atts) );

		$entry = null;
		if ( 'default' !== $id ) {
			$entry = get_post( $id );
		}
		if ( ! $entry ) {
			if ( 'default' !== $slug ) {
				$search = get_posts( array(
					'post_type' => $this->dictionary->slug(),
					'post_status' => 'publish',
					'numberposts' => 1,
					'name' => $slug,
				) );
				if ( $search ) {
					$entry = get_post( $search[0]->ID );
				}
			}
		}
		if ( ! $entry ) {
			$error = '<p>' . __( 'Entry not available.', 'dictionary-pro' ) . '</p>';
		}

		if ( $entry && 'title' === strtolower( $display ) ) {
			$display = get_the_title( $entry );
		} elseif ( $entry ) {
			   $display = apply_filters( 'dictionary_format_entry', $entry, true );
		} else {
			$error = '<p>' . __( 'Entry not available.', 'dictionary-pro' ) . '</p>';
		}
		if ( isset( $error ) ) {
			$display = $error;
		}
		return $display;
	}

	public function link( $atts ) {
		extract( shortcode_atts( array(
			'id' => 'default',
			'slug' => 'default',
			'text' => 'default',
		), $atts) );

		// Check for no ID, but a slug
		if ( 'default' === $id and 'default' !== $slug  ) {
			$wpd = get_page_by_path( $slug, 'OBJECT', 'dictionary' );
			if ( $wpd ) {
				$id = $wpd->ID;
			} else {
				$id = 'default';
			}
		}

		// Now that we have an ID, regardless of slug or ID provided...
		$entry = get_post( $id );

		if ( ! empty( $entry ) ) {
			// If the text isn't default, then use the dictionary entry title
			if ( 'default' !== $text ) {
				$link_text = $text;
			} else {
				$link_text = get_the_title( $entry->ID );
			}

			// Now, for the title content
			$entry_link = apply_filters( 'dictionary_entry_link', $entry );
			$link = str_replace( '>$1<', ">$link_text<", $entry_link );

			return $link;
		} else {
			return null;
		}
	}
}
